@@ -93,6 +93,7 @@ i_t1_new(char *pfb,char *afm) {
   font_id = T1_AddFont(pfb);
   if (font_id<0) {
     mm_log((1,"i_t1_new: Failed to load pfb file '%s' - return code %d.\n",pfb,font_id));
+    t1_push_error();
     return font_id;
   }
   
@@ -101,8 +102,18 @@ i_t1_new(char *pfb,char *afm) {
     if (T1_SetAfmFileName(font_id,afm)<0) mm_log((1,"i_t1_new: afm loading of '%s' failed.\n",afm));
   }
 
+  if (T1_LoadFont(font_id)) {
+    mm_log((1, "i_t1_new() -> -1 - T1_LoadFont failed (%d)\n", T1_errno));
+    t1_push_error();
+    i_push_error(0, "loading font");
+    T1_DeleteFont(font_id);
+    return -1;
+  }
+
   ++t1_active_fonts;
 
+  mm_log((1, "i_t1_new() -> %d\n", font_id));
+
   return font_id;
 }
 
@@ -572,6 +583,10 @@ i_t1_glyph_name(int font_num, unsigned long ch, char *name_buf,
 
 static void
 t1_push_error(void) {
+#if T1LIB_VERSION > 5 || T1LIB_VERSION == 5 && T1LIB_VERSION >= 1
+  /* I don't know when T1_StrError() was introduced, be conservative */
+  i_push_error(T1_errno, T1_StrError(T1_errno));
+#else
   switch (T1_errno) {
   case 0: 
     i_push_error(0, "No error"); 
@@ -579,108 +594,115 @@ t1_push_error(void) {
 
 #ifdef T1ERR_SCAN_FONT_FORMAT
   case T1ERR_SCAN_FONT_FORMAT:
-    i_push_error(T1ERR_SCAN_FONT_FORMAT, "SCAN_FONT_FORMAT"); 
+    i_push_error(T1ERR_SCAN_FONT_FORMAT, "Attempt to Load Multiple Master Font"); 
     break;
 #endif
 
 #ifdef T1ERR_SCAN_FILE_OPEN_ERR
   case T1ERR_SCAN_FILE_OPEN_ERR:
-    i_push_error(T1ERR_SCAN_FILE_OPEN_ERR, "SCAN_FILE_OPEN_ERR"); 
+    i_push_error(T1ERR_SCAN_FILE_OPEN_ERR, "Type 1 Font File Open Error"); 
     break;
 #endif
 
 #ifdef T1ERR_SCAN_OUT_OF_MEMORY
   case T1ERR_SCAN_OUT_OF_MEMORY:
-    i_push_error(T1ERR_SCAN_OUT_OF_MEMORY, "SCAN_OUT_OF_MEMORY"); 
+    i_push_error(T1ERR_SCAN_OUT_OF_MEMORY, "Virtual Memory Exceeded"); 
     break;
 #endif
 
 #ifdef T1ERR_SCAN_ERROR
   case T1ERR_SCAN_ERROR:
-    i_push_error(T1ERR_SCAN_ERROR, "SCAN_ERROR"); 
+    i_push_error(T1ERR_SCAN_ERROR, "Syntactical Error Scanning Font File"); 
     break;
 #endif
 
 #ifdef T1ERR_SCAN_FILE_EOF
   case T1ERR_SCAN_FILE_EOF:
-    i_push_error(T1ERR_SCAN_FILE_EOF, "SCAN_FILE_EOF"); 
+    i_push_error(T1ERR_SCAN_FILE_EOF, "Premature End of Font File Encountered"); 
     break;
 #endif
 
 #ifdef T1ERR_PATH_ERROR
   case T1ERR_PATH_ERROR:
-    i_push_error(T1ERR_PATH_ERROR, "PATH_ERROR"); 
+    i_push_error(T1ERR_PATH_ERROR, "Path Construction Error"); 
     break;
 #endif
 
 #ifdef T1ERR_PARSE_ERROR
   case T1ERR_PARSE_ERROR:
-    i_push_error(T1ERR_PARSE_ERROR, "PARSE_ERROR"); 
+    i_push_error(T1ERR_PARSE_ERROR, "Font is Corrupt"); 
     break;
 #endif
 
 #ifdef T1ERR_TYPE1_ABORT
   case T1ERR_TYPE1_ABORT:
-    i_push_error(T1ERR_TYPE1_ABORT, "TYPE1_ABORT"); 
+    i_push_error(T1ERR_TYPE1_ABORT, "Rasterization Aborted"); 
     break;
 #endif
 
 #ifdef T1ERR_INVALID_FONTID
   case T1ERR_INVALID_FONTID:
-    i_push_error(T1ERR_INVALID_FONTID, "INVALID_FONTID"); 
+    i_push_error(T1ERR_INVALID_FONTID, "Font ID Invalid in this Context"); 
     break;
 #endif
 
 #ifdef T1ERR_INVALID_PARAMETER
   case T1ERR_INVALID_PARAMETER:
-    i_push_error(T1ERR_INVALID_PARAMETER, "INVALID_PARAMETER"); 
+    i_push_error(T1ERR_INVALID_PARAMETER, "Invalid Argument in Function Call"); 
     break;
 #endif
 
 #ifdef T1ERR_OP_NOT_PERMITTED
   case T1ERR_OP_NOT_PERMITTED:
-    i_push_error(T1ERR_OP_NOT_PERMITTED, "OP_NOT_PERMITTED"); 
+    i_push_error(T1ERR_OP_NOT_PERMITTED, "Operation not Permitted"); 
     break;
 #endif
 
 #ifdef T1ERR_ALLOC_MEM
   case T1ERR_ALLOC_MEM:
-    i_push_error(T1ERR_ALLOC_MEM, "ALLOC_MEM"); 
+    i_push_error(T1ERR_ALLOC_MEM, "Memory Allocation Error"); 
     break;
 #endif
 
 #ifdef T1ERR_FILE_OPEN_ERR
   case T1ERR_FILE_OPEN_ERR:
-    i_push_error(T1ERR_FILE_OPEN_ERR, "FILE_OPEN_ERR"); 
+    i_push_error(T1ERR_FILE_OPEN_ERR, "Error Opening File"); 
     break;
 #endif
 
 #ifdef T1ERR_UNSPECIFIED
   case T1ERR_UNSPECIFIED:
-    i_push_error(T1ERR_UNSPECIFIED, "UNSPECIFIED"); 
+    i_push_error(T1ERR_UNSPECIFIED, "Unspecified T1Lib Error"); 
     break;
 #endif
 
 #ifdef T1ERR_NO_AFM_DATA
   case T1ERR_NO_AFM_DATA:
-    i_push_error(T1ERR_NO_AFM_DATA, "NO_AFM_DATA"); 
+    i_push_error(T1ERR_NO_AFM_DATA, "Missing AFM Data"); 
     break;
 #endif
 
 #ifdef T1ERR_X11
   case T1ERR_X11:
-    i_push_error(T1ERR_X11, "X11"); 
+    i_push_error(T1ERR_X11, "X11 Interface Error"); 
     break;
 #endif
 
 #ifdef T1ERR_COMPOSITE_CHAR
   case T1ERR_COMPOSITE_CHAR:
-    i_push_error(T1ERR_COMPOSITE_CHAR, "COMPOSITE_CHAR"); 
+    i_push_error(T1ERR_COMPOSITE_CHAR, "Missing Component of Composite Character"); 
+    break;
+#endif
+
+#ifdef T1ERR_SCAN_ENCODING
+  case T1ERR_SCAN_ENCODING:
+    i_push_error(T1ERR_SCAN_ENCODING, "Error Scanning Encoding File"); 
     break;
 #endif
 
   default:
     i_push_errorf(T1_errno, "unknown error %d", (int)T1_errno);
   }
+#endif
 }
 
