@@ -1,7 +1,6 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#define IMAGER_NO_CONTEXT
 #include "imager.h"
 
 /*
@@ -130,9 +129,8 @@ llist_push(struct llist *l,const void *data) {
     }
   }
   /*   fprintf(stderr,"0x%08X\n",l->t); */
-  if (llist_llink_push(l,l->t,data)) {
-    dIMCTX;
-    im_fatal(aIMCTX, 3, "out of memory\n");
+  if (llist_llink_push(l,l->t,data)) { 
+    i_fatal(3, "out of memory\n");
   }
 }
 
